;******************************************************************************
;******************************************************************************
;******************************************************************************
;              ULOHA YD14SAP - 2009/2010 - SYS/ISA PIC18 - CITAC - DIR
;******************************************************************************
;******************************************************************************
;******************************************************************************
    ;XTAL = 10MHz
	LIST P=18F87J11				; vyber CPU
	#include <P18F87J11X.INC>	; hlavickovy soubor pro tento CPU

;******************************************************************************
; Konfigurace CPU
;******************************************************************************
    CONFIG	FOSC = HS           ; HS oscillator
    CONFIG  XINST = OFF         ; Rozsirena sada instrukci NE
    CONFIG  WDTEN = OFF         ; Watchdog NE
    CONFIG  IESO = OFF          ; Dvourychlostni start NE
;******************************************************************************
; Ovladaci tlacitka
;******************************************************************************
#define STOPSTART   PORTB,0     ; Tlacitko
#define DIRES       PORTA,5     ; Tlacitko
#define Aktivni_Uroven_L        ; Chovani tlacitka
;******************************************************************************
; Konstanty programu
;******************************************************************************
K2      EQU     .11              ; Konstanty polynomu
K1      EQU     .6
K0      EQU     .12
;******************************************************************************
T0H     EQU     0x67            ; predvolba pro T0 MSB 500 ms
T0L     EQU     0x69            ; predvolba pro T0 LSB
T1H     EQU     0x9E            ; predvolba pro T1 MSB 10 ms
T1L     EQU     0x64            ; predvolba pro T1 LSB
TTLAC   EQU     .25             ; filtracni cas pro tlacitka [*10ms]
;******************************************************************************
; Definuj nejake pomenne
;******************************************************************************
                CBLOCK	0x000   ; nastaven konstant
;******************************************************************************
Counter          				; 8bit citac
clip            				; citac pro obsluhu tlacitek
Remote      	    			; ovladaci bity programu
RESULT:3                        ; Vysledek 24 bit
AOP:3                           ; Pomocny 24 operand
XnaDruhou:2                     ; X^2 16 bit Pomocna promena
WTEMP    	                    ; ulozny registr pro WREG
STEMP   	                    ; ulozny registr pro STATUS
BTEMP   	                    ; ulozny registr pro BSR (je-li treba)
;******************************************************************************
                ENDC
;******************************************************************************
; Definuj ovladaci bity programu
;******************************************************************************
#define StopRun     Remote,0
#define UpDn        Remote,1
#define ChangeFlag  Remote,2
;******************************************************************************
#ifdef  Aktivni_Uroven_L
#define TEST        btfsc       ; Tlacitko aktivni v L
#else 
#define TEST        btfss       ; Tlacitko aktivni v H
#endif
;******************************************************************************
;******************************************************************************
;******************************************************************************
;******************************************************************************
;                             ZACATEK ASM KODU
;******************************************************************************
;******************************************************************************
;******************************************************************************
;******************************************************************************


;******************************************************************************
; Reset vektor  -- dle dokumentace
;******************************************************************************
		ORG	0x0000
		goto	Program		;Start programu
;******************************************************************************
; Vektor pro preruseni s vysokou prioritou
;******************************************************************************
		ORG	0x0008
		goto	T0CODE		;Obsluha T0 / vysoka priorita
;******************************************************************************
; Vektor pro preruseni s nizkou prioritou
;******************************************************************************
		ORG	0x0018
        goto    T1CODE      ;Obsluha T1 / nizka priorita
;******************************************************************************






;******************************************************************************
;******************************************************************************
;******************************************************************************
;***************************** HLAVNI PROGRAM *********************************
;******************************************************************************
;******************************************************************************
;******************************************************************************
Program: 

        clrf    BSR                     ; bank selektor ukazuje na banku 0x00
									    ; symbolicke jmeno
        
;******************************************************************************
; Vychozi nastaveni programu
;******************************************************************************
        bsf     ChangeFlag              ; aktualizuj hodnoty 1s po startu
        bcf     StopRun                 ; citac po startu stoji
        bsf     UpDn                    ; citani nahoru
        clrf    Counter                 ; vynuluj citac / neni nutne
        clrf    clip                    ; vynuluj filtracni citac / neni nutne
        call    CLRRES                  ; vynuluj RESULT / neni nutne
        call    CLRAOP                  ; vynuluj AOP / neni nutne

;******************************************************************************
; Nastaveni I/O t.j. vystup pro citac a vstupy pro tlacitka
;******************************************************************************
        
        ; Vsechno vstupy
        movlw   b'11111111'
        movwf   TRISA
        
        ; Port RA konfigurovat jako digitalni I/O
        BSF WDTCON, ADSHR 
        MOVLW 0xFF
        MOVWF ANCON0
        BCF WDTCON, ADSHR

        ; Vsechno vstupy
        movlw   b'11111111'
        movwf   TRISB

        ; Vsechno vystupy
        movlw   b'00000000'
        movwf   TRISD
        movlw   b'11111111'             ; Test ledek zda sviti
        movwf   PORTD

;******************************************************************************
; Nastaveni casovace T0   
;******************************************************************************
        
        ; Povolit preddelic pro T0
        bcf     T0CON, PSA

        ; Preddelic pro T0=32 ////////// 100 = 1:32 Prescale value
        bsf     T0CON, T0PS2 
        bcf     T0CON, T0PS1
        bcf     T0CON, T0PS0

        ; T0 bude pracovat jako 16bit
        bcf     T0CON, T08BIT

        ; Zdroj hodin pro T0 bude interni (sysclock= XTAL / 4)
        bcf     T0CON, T0CS

        ;Prednastavit citac T0
        movlw   T0H
        movwf   TMR0H
        movlw   T0L
        movwf   TMR0L
        
        ; Spustit citani
        bsf     T0CON, TMR0ON

;******************************************************************************
; Nastaveni casovace T1
;******************************************************************************
        
        ; povolit 16bit operace na T1
        bsf     T1CON, RD16

        ; zdroj hodin pro T1 z jineho zdroje nez oscilator
        bcf     T1CON, T1RUN

        ; nastaveni preddelice pro T1 na hodnotu 1
        bcf     T1CON, T1CKPS1
        bcf     T1CON, T1CKPS0

        ; timer1 oscilotor vypnut
        bcf     T1CON, T1OSCEN

        ; synchlonizace s externim vstupem vypnuta
        bsf     T1CON, T1SYNC

        ; zdroj hodin pro T1 bude interni (sysclock= XTAL/4)
        bcf     T1CON, TMR1CS

        ;Prednastavit citac T1
        movlw   T1H
        movwf   TMR1H
        movlw   T1L
        movwf   TMR1L
        
        ; Spustit citani
        bsf     T1CON, TMR1ON

;******************************************************************************
; Nastaveni preruseni a jejich priorit pro jednotlive zdroje T0/T1
;******************************************************************************
        
        ; Dve urovne priorit povoleny
        bsf     RCON, IPEN
        
        ; Povolit IRQ od T0
        ; Zvolit prioritu IRQ_T0
        bsf     INTCON, TMR0IE
        bsf     INTCON2, TMR0IP


        ; Povolit IRQ od T1
        ; Zvolit prioritu IRQ_T1
        bsf     PIE1, TMR1IE
        bcf     IPR1, TMR1IP


        ; Povolit preruseni s vysokou urovni
        ; Povolit preruseni s nizkou urovni
        bsf     INTCON, GIEH
        bsf     INTCON, GIEL

;******************************************************************************
; Nekonecna smycka - CPU ceka na interrupt
;******************************************************************************
cekej:   goto   cekej
;******************************************************************************
;******************************************************************************
;******************************************************************************
;****************************** OBSLUHA T0 ************************************
;******************************************************************************
;******************************************************************************
T0CODE: movlw   T0L                 ; Prednastavit citac T0
        movwf   TMR0L         

; Cita se pouze nahoru

        btfss   StopRun
        goto    nuluj
        incf    Counter
        bsf     ChangeFlag
;******************************************************************************
nuluj:  btfss   UpDn
        goto    change
        bcf     UpDn
        clrf    Counter
        bsf     ChangeFlag 
;******************************************************************************
; y=K2*x^2+K1*x+K0        
;******************************************************************************
change: btfss   ChangeFlag
        goto    stuj
        bcf     ChangeFlag
        call    CLRRES
        call    PLNM2   
        call    PLNM1
        call    PLNM0
;******************************************************************************
        movff   Counter, PORTD      ; zobrazit citac na LEDKY
;******************************************************************************
stuj:   bcf     INTCON, TMR0IF      ; smazat priznak preruseni
        retfie	FAST                ; navrat z preruseni
;******************************************************************************





;******************************************************************************
;*************************** Podprogramy pro T0 *******************************
;******************************************************************************


;******************************************************************************
; RESULT=RESULT+K0
;******************************************************************************
PLNM0:  call    CLRAOP
        movlw   K0
        movwf   AOP+0
        call    RPLUSA
        return
;******************************************************************************
; RESULT=RESULT+K1*x
;******************************************************************************
PLNM1:  call    CLRAOP
        movlw   K1
        mulwf   Counter
        movff   PRODH,AOP+1
        movff   PRODL,AOP+0
        call    RPLUSA
        return
;******************************************************************************
; RESULT=RESULT+K2*x^2
;******************************************************************************
PLNM2:  call    XNA2
        call    CLRAOP
        movlw   K2
        mulwf   XnaDruhou+0
        movff   PRODH,AOP+1
        movff   PRODL,AOP+0
        call    RPLUSA
        call    CLRAOP
        movlw   K2
        mulwf   XnaDruhou+1
        movff   PRODH,AOP+2
        movff   PRODL,AOP+1
        call    RPLUSA
        return
;******************************************************************************
; x=Counter^2
;******************************************************************************
XNA2:   movf    Counter,W
        mulwf   Counter
        movff   PRODH,XnaDruhou+1
        movff   PRODL,XnaDruhou+0
        return
;******************************************************************************
; RESULT=0
;******************************************************************************
CLRRES: clrf    RESULT+2
        clrf    RESULT+1
        clrf    RESULT+0
        return
;******************************************************************************
; AOP=0
;******************************************************************************
CLRAOP: clrf    AOP+2
        clrf    AOP+1
        clrf    AOP+0
        return
;******************************************************************************
; RESULT=RESULT+AOP
;******************************************************************************
RPLUSA: movf    AOP+0,W
        addwf   RESULT+0,F
        movf    AOP+1,W
        addwfc  RESULT+1,F
        movf    AOP+2,W
        addwfc  RESULT+2,F
        return
;******************************************************************************









;******************************************************************************
;******************************************************************************
;****************************** OBSLUHA T1 ************************************
;******************************************************************************
;******************************************************************************
T1CODE: movff	STATUS, STEMP    	; zapamatuj stav registru STATUS
		movff	WREG, WTEMP  		; zapamatuj stav registru WREG
		movff	BSR, BTEMP	    	; zapamatuj stav registru BSR
;******************************************************************************
        movlw   T1L                 ; prednastavit citac T1
        movwf   TMR1L
;******************************************************************************
        TEST    STOPSTART           ; tlacitko START/STOP ?
        goto    skip00
        tstfsz  clip
        goto    hold00
        btg     StopRun
hold00: movlw   TTLAC
        movwf   clip
        goto    skip02
;******************************************************************************
skip00: TEST    DIRES               ; tlacitko DIR/RES ?
        goto    skip01
        tstfsz  clip
        goto    hold01
        btg     UpDn
hold01: movlw   TTLAC
        movwf   clip
        goto    skip02
;******************************************************************************
skip01: tstfsz  clip                ;filtracni citac
        decf    clip,F
;******************************************************************************
skip02: movff	BTEMP, BSR		    ; vrat zpet hodnotu registru BSR
		movff	WTEMP, WREG		    ; vrat zpet hodnotu registru WREG
		movff	STEMP, STATUS	    ; vrat zpet hodnotu registru STATUS
        bcf     PIR1, TMR1IF        ; smazat priznak preruseni
		retfie                      ; navrat z preruseni
;******************************************************************************







;******************************************************************************
;******************************************************************************
;******************************************************************************
; Konec programu
;******************************************************************************
		END
